%% Test RAI structure learning algorithm

%% load Alarm network and training dataset
load alarm_bnt_qian_diao;
% bnet_Alarm = mk_alarm_bnet; % ground truth
AlarmTraining = load('alarm10000.mat'); % training dataset
% AlarmTraining.data = load('alarm.mat', '-ascii')'; % training dataset
[n_samples,n_nodes] = size(AlarmTraining.data);

%% Example for threshold calculation
% This part provides visualization for threshold selection heuristic
SortedMI = MIStats(AlarmTraining.data);

Th = 3e-3; % obtained from the ploted graphs

%% Learn Alarm network structure using RAI
Initial_Graph = setdiag(ones(n_nodes), 0); % fully connected graph
Initial_DSepSets = cell(n_nodes);

tic
[pdag_rai, seps, Nci_rai] = ...
    learn_struct_pdag_rai(Initial_Graph,[],1:n_nodes,Initial_DSepSets,0,'mutualC_f_e',AlarmTraining.data,Th);

elapsed_RAI = toc;

%% Runtime and complexity performance
% Complexity
figure;
bar(0:7,Nci_rai(1:8)); % number of conditional independence tests of each order
grid on
xlabel('condition-set size (order of CI test');
ylabel('number of CI tests');
title('Number of CI tests and orders learning Alarm network using RAI');
% Runtime
fprintf('\n--------------------------------------------------------------------\\\n');
fprintf('RAI structure learning results for Alarm network (10,000 samples)\n\n');
fprintf('Runtime and complexity\n');
fprintf('~~~~~~~~~~~~~~~~~~~~~~\n');
fprintf('RAI Alarm network learning time: %.1f sec\n\n',elapsed_RAI);
fprintf('Number of CI tests:\n   orders 0-3:\t%d\n   orders 4-35:\t%d\n',sum(Nci_rai(1:4)),sum(Nci_rai(5:end)));

%% Calculate structural Hamming distance errors
[SHD_RAI, SHD_u_RAI, SHD_d_RAI, SHD_u_err_RAI, SHD_d_err_RAI] = struct_hamming_dist(pdag_rai, bnet_Alarm.dag);
fprintf('\nStructural Hamming distance\n');
fprintf('~~~~~~~~~~~~~~~~~~~~~~~~~~~\n');
fprintf('   Missing edges:\t%d\n',SHD_u_err_RAI.omissions);
fprintf('   Extra edges:\t%d\n',SHD_u_err_RAI.commissions);
fprintf('   Total skeleton learning error:\t%d\n',SHD_u_RAI);
fprintf('\n   Missing edge directions:\t%d\n',SHD_d_err_RAI.omissions);
fprintf('   Extra edge directions:\t%d\n',SHD_d_err_RAI.commissions);
fprintf('   Reversed edge directions:\t%d\n',SHD_d_err_RAI.reversed);
fprintf('   Total edge directionality learning error:\t%d\n',SHD_d_RAI);
fprintf('\n   Total structural Hamming distance (skeleton + directionality): %d\n',SHD_RAI);
fprintf('--------------------------------------------------------------------/\n\n');
